#| -*-Scheme-*-

Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015,
    2016, 2017, 2018, 2019 Matthew Birkholz

This file is part of a planetarium plugin for MIT/GNU Scheme Pucked.

This plugin is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This plugin is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this plugin; if not, write to the Free Software Foundation,
Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

|#

;;; Much of this is stolen from Arthur's WorldMap.java, which he
;;; mistakenly lent me.

(define (draw-earth device orientation solar-lat/long)
  (let ((solar-orientation
	 (let ((Mx (make-x-rotation-matrix
		    (degrees->radians
		     (flo:negate (latitude solar-lat/long)))))
	       (My (make-y-rotation-matrix
		    (degrees->radians (longitude solar-lat/long)))))
	   (3d-multiply! orientation My My)
	   (3d-multiply! My Mx My)
	   My))
	(can-fill? (fill-polygon-available?)))
    (if can-fill?
	(draw-dark-side device orientation solar-lat/long solar-orientation))
    (draw-meridians device orientation)
    (draw-parallels device orientation)
    (draw-coastlines device orientation)
    (if (not can-fill?)
	(draw-terminator device solar-orientation))
    (draw-circle device (make-2d-point 0. 0.) 1. "black")))

(define (draw-meridians device orientation)
  (draw-meridian device 0. orientation "green")
  (let loop ((i 1.))
    (if (flo:< i 36.)			;every 10 degrees
	(let ((longitude (flo:* i (flo:/ 2pi 36.))))
	  (draw-meridian device longitude orientation "gray")
	  (loop (flo:+ i 1.))))))

(define (draw-meridian device longitude orientation color)
  (let loop ((i -8.)
	     (start (project -pi/2 longitude orientation)))
    (if (flo:<= i 9.)			;every 10 degrees
	(let* ((latitude (flo:* i (flo:/ pi 18.)))
	       (end (project latitude longitude orientation)))
	  (if (and (not (flo:negative? (z start)))
		   (not (flo:negative? (z end))))
	      (draw-segment device start end color))
	  (loop (flo:+ i 1.) end)))))

(define (draw-parallels device orientation)
  (draw-parallel device 0. orientation "blue")
  (let* ((tropic (degrees->radians (dms->degrees 23. 26. 16.)))
	 (arctic (flo:- pi/2 tropic)))
    (draw-parallel device tropic orientation "blue")
    (draw-parallel device (flo:negate tropic) orientation "blue")
    (draw-parallel device arctic orientation "blue")
    (draw-parallel device (flo:negate arctic) orientation "blue"))
  (let loop ((i 1.))
    (if (flo:< i 9.)			;every 10 degrees
	(let ((latitude (flo:* i (flo:/ pi/2 9.))))
	  (draw-parallel device (flo:negate latitude) orientation "gray")
	  (draw-parallel device latitude orientation "gray")
	  (loop (flo:+ i 1.))))))

(define (draw-parallel device latitude orientation color)
  (let loop ((i 1.)
	     (start (project latitude 0. orientation)))
    (if (flo:<= i 36.)			;every 10 degrees
	(let* ((longitude (flo:* i (flo:/ 2pi 36.)))
	       (end (project latitude longitude orientation)))
	  (if (and (not (flo:negative? (z start)))
		   (not (flo:negative? (z end))))
	      (draw-segment device start end color))
	  (loop (flo:+ i 1.) end)))))

(define (project latitude longitude orientation)
  (let ((point (make-3d-point
		(flo:* (flo:cos latitude) (flo:sin longitude))
		(flo:sin latitude)
		(flo:* (flo:cos latitude) (flo:cos longitude)))))
    (3d-transform! point orientation point)
    point))

(define (draw-coastlines device orientation)
  (draw-cil device africa-cil orientation "black")
  (draw-cil device asia-cil orientation "black")
  (draw-cil device europe-cil orientation "black")
  (draw-cil device namer-cil orientation "black")
  (draw-cil device samer-cil orientation "black"))

(define (draw-cil device cil orientation color)
  (for-each (lambda (stroke)
	      (let loop ((start (project (caar stroke) (cdar stroke)
					 orientation))
			 (rest (cdr stroke)))
		(if (pair? rest)
		    (let ((end (project (caar rest) (cdar rest) orientation)))
		      (if (and (not (flo:negative? (z start)))
			       (not (flo:negative? (z end))))
			  (draw-segment device start end color))
		      (loop end (cdr rest))))))
	    cil))

(define (draw-terminator device sol-orientation)
  ;;(graphics-set-line-width! device 4.0)
  (draw-meridian device pi/2 sol-orientation "black")
  (draw-meridian device -pi/2 sol-orientation "black"))

(define (draw-dark-side device orientation sol sol-orientation)

  (define (find-point found? start-i)
    ;; Returns an exact index ranging from -18.0 to +17.0.  START-I
    ;; has the same range.  The index is negative while we search up
    ;; the back (longitude -pi/2, latitudes from -pi/2 to pi/2) and
    ;; positive while we search down the front (longitude pi/2,
    ;; latitudes from pi/2 to -pi/2).
    (if start-i
	(let loop ((i start-i))
	  (let ((projection (project-i i)))
	    (if (found? projection)
		i
		(let ((i+1 (next-i i)))
		  (if (flo:= i+1 start-i)
		      #f
		      (loop i+1))))))
	#f))

  (define-integrable (project-i i)
    (let ((latitude (if (flo:negative? i)
			(flo:* (flo:+ i 9.) (flo:/ pi/2 9.))
			(flo:* (flo:- 9. i) (flo:/ pi/2 9.))))
	  (longitude (if (flo:negative? i) -pi/2 pi/2)))
      (project latitude longitude sol-orientation)))

  (define-integrable (next-i i)
    (let ((i+1 (flo:+ i 1.)))
      (if (flo:< i+1 18.) i+1 -18.)))

  (define (hidden? point)
    (flo:negative? (z point)))

  (define (visible? point)
    (not (flo:negative? (z point))))

  (define (collect-visible-reversed start-i)
    (let loop ((points (list (project-i start-i)))
	       (i (next-i start-i)))
      (if (flo:= i start-i)
	  points
	  (let ((projection (project-i i)))
	    (if (visible? projection)
		(loop (cons projection points) (next-i i))
		points)))))

  (define (circle-points start end)
    (let ((increment (flo:/ pi 18.)))
      (let ((stop (greater-angle start end)))
	(let loop ((a start))
	  (if (flo:< a stop)
	      (cons (flo:2d (flo:cos a) (flo:sin a))
		    (loop (flo:+ a increment)))
	      '())))))

  (let ((start-i (find-point visible? (find-point hidden? -18.))))
    (if start-i
	(let ((points (collect-visible-reversed start-i)))
	  (let ((end-point (car points)))
	    (let ((points (reverse! points)))
	      #;(fluid-let ((flonum-unparser-cutoff '(relative 5)))
		(trace "; terminator's visible points:\n")
		(for-each (lambda (p)
			    (trace "; 	"(x p)","(y p)"\n"))
			  points))
	      (let ((start-point (car points)))
		(let ((edge
		       (circle-points
			(flo:atan2 (y end-point) (x end-point))
			(flo:atan2 (y start-point)(x start-point)))))
		  #;(fluid-let ((flonum-unparser-cutoff '(relative 5)))
		    (trace "; disk edge:\n")
		    (for-each (lambda (p)
				(trace "; 	"(x p)","(y p)"\n"))
			      edge))
		  (fill-polygon device (append! points edge) "light gray"))))))
	(let ((sol-point (project (degrees->radians (latitude sol))
				  (degrees->radians (longitude sol))
				  orientation)))
	  (if (hidden? sol-point)
	      (fill-polygon device (circle-points 0. 2pi) "light gray"))))))