#!/bin/sh
# -*-Scheme-*-
#
# Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015,
#     2016, 2017, 2018, 2019 Matthew Birkholz
# 
# This file is part of a planetarium plugin for MIT/GNU Scheme Pucked.
# 
# This plugin is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This plugin is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this plugin; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301, USA.

# Compile the PLANETARIUM option.

set -e
: ${MIT_SCHEME_EXE=mit-scheme}
${MIT_SCHEME_EXE} --batch-mode <<\EOF
(begin

  (define (cf files deps pkg-name)
    (if (pair? files)
	(for-each (lambda (f) (compile-file f deps (->environment pkg-name)))
		  files)
	(compile-file files deps (->environment pkg-name))))

  (define (lf file pkg-name)
    (load file (->environment pkg-name)))

  (parameterize ((param:suppress-loading-message? #t))
    (load-option 'cref)
    (load-option 'gtk))

  (if (name->package '(planetarium))
      (error "The planetarium package already exists."))
  (let ((package-set (package-set-pathname "mit")))
    (if (not (file-modification-time<? "mit.pkg" package-set))
	(cref/generate-trivial-constructor "mit"))
    (construct-packages-from-file (fasload package-set)))

  (cf "mit-r3rs"     '() '(r3rs extras))
  (lf "mit-r3rs"         '(r3rs extras))
  (cf "mit-syntax"   '() '(planetarium syntax))
  (lf "mit-syntax"       '(planetarium syntax))
  (cf '("geometry" "matrices" "time" "graphics"
	"solar")     '() '(planetarium))
  (cf '("earth" "tellurion")
      '("geometry" "matrices") '(planetarium))
  (cf "mit-gtk"      '() '(planetarium gtk-graphics))
  (cf "mit-cairo"    '() '(planetarium cairo-graphics))
  (cf "mit-x"        '() '(planetarium x-graphics))
  (cf "mit-graphics" '() '(planetarium simple-graphics))
  (cf "mit-cil"      `("mit-syntax" ,@(directory-read "cil-*.txt"))
      '(planetarium earth-cil))

  (cref/generate-constructors "mit")

  (if (not (warn-errors?
	    (lambda ()
	      (parameterize ((param:suppress-loading-message? #t))
		(load-option 'gl)))))
      (begin
	(let ((package-set (package-set-pathname "mit-3d")))
	  (if (not (file-modification-time<? "mit-3d.pkg" package-set))
	      (cref/generate-trivial-constructor "mit-3d"))
	  (construct-packages-from-file (fasload package-set)))

	(cf "terrain"      '("geometry") '(planetarium terrain))
	(cf "google-earth" '() '(planetarium google-earth))

	(cref/generate-constructors "mit-3d")))
  )
EOF
suffix=`echo "(display (microcode-id/operating-system-suffix))" \
	| ${MIT_SCHEME_EXE} --batch-mode`
report=mit-$suffix.crf
if [ -s "$report" ]; then
    awk "{printf \"$report:%d: %s\\n\",NR,\$0}" $report
fi
report=mit-3d-$suffix.crf
if [ -s "$report" ]; then
    awk "{printf \"$report:%d: %s\\n\",NR,\$0}" $report
fi
