#| -*-Scheme-*-

Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015,
    2016, 2017, 2018, 2019 Matthew Birkholz

This file is part of a planetarium plugin for MIT/GNU Scheme Pucked.

This plugin is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This plugin is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this plugin; if not, write to the Free Software Foundation,
Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

|#

(define-syntax define-integrable-operator
  ;; Like define-integrable except it does not integrate the parameters.
  (rsc-macro-transformer
   (lambda (form environment)
     (declare (ignore environment))
     (if (syntax-match? '((identifier . mit-bvl) + form) (cdr form))
	 `(begin
	   (declare (integrate-operator ,(caadr form)))
	   (define ,@(cdr form)))
	 (ill-formed-syntax form)))))

(define-syntax cil-file
  (sc-macro-transformer
   (lambda (form usage-env)
     (declare (ignore usage-env))
     (let ((filename (cadr form)))
       (list 'quote (read-cil-file filename))))))

(define-integrable 2pi (flo:* 8. (flo:atan2 1. 1.)))
(define-integrable (degrees->radians degrees) (flo:* degrees (flo:/ 2pi 360.)))

(define (read-cil-file name)
  (let ((in (open-input-file name)))
    (port/set-line-ending in 'crlf)
    (let loop ((points '()) (lines '()))
      (let ((line (read-line in)))
	(if (eof-object? line)
	    (begin
	      (close-port in)
	      (reverse! (if (null? points)
			    lines
			    (cons points lines))))
	    (let ((line (string-trim-left line)))
	      (cond
	       ((string-null? line)
		(loop points lines))
	       ((string=? line "-1")
		(loop '() (cons points lines)))
	       (else
		(let ((sep (string-find-next-char line #\space)))
		  (let ((latitude (string->number (string-head line sep)))
			(longitude (string->number (string-tail line (1+ sep)))))
		    (if (or (not latitude) (not longitude))
			(error "could not parse:" (input-port/line in) line))
		    (loop (cons (cons (degrees->radians
				       (flo:/ (->flonum latitude) 3600.))
				      (degrees->radians
				       (flo:/ (->flonum longitude) 3600.)))
				points)
			  lines)))))))))))